Package: HKRbook
Type: Package
Title: Apps and Data for the Book "Introduction to Statistics"
Version: 0.1.3
Authors@R: person("Sigbert", "Klinke", email="sigbert@hu-berlin.de", role=c("aut", "cre"))
Description: Functions, Shiny apps and data for the book "Introduction to Statistics" by
   Wolfgang Karl Härdle, Sigbert Klinke, and Bernd Rönz (2015) <doi:10.1007/978-3-319-17704-5>.
License: GPL-3
Encoding: UTF-8
Depends: tools
Suggests: cluster, knitr, magrittr, rmarkdown
Imports: MASS, shiny, shinydashboard, shinyWidgets, shinydashboardPlus,
        DT, highlight, formatR, scatterplot3d
RoxygenNote: 7.2.1
VignetteBuilder: knitr
NeedsCompilation: no
Packaged: 2022-10-06 05:53:37 UTC; sk
Author: Sigbert Klinke [aut, cre]
Maintainer: Sigbert Klinke <sigbert@hu-berlin.de>
Repository: CRAN
Date/Publication: 2022-10-06 06:20:02 UTC
Built: R 4.2.0; ; 2022-10-06 21:09:20 UTC; unix
