# Generated by roxygen2: do not edit by hand

S3method(html_matrix,default)
S3method(toHTML,html_matrix)
S3method(toHTML,matrix)
S3method(toHTML,table)
export(checkPackages)
export(distributionParams)
export(gettext)
export(hm_cell)
export(hm_col)
export(hm_colmargin)
export(hm_colmargintitle)
export(hm_index)
export(hm_row)
export(hm_rowmargin)
export(hm_rowmargintitle)
export(hm_table)
export(hm_title)
export(hm_total)
export(hm_tr)
export(htmlTable)
export(html_matrix)
export(in_range)
export(is.ASCII)
export(men_asso)
export(men_bin)
export(men_ci1)
export(men_ci2)
export(men_cilen)
export(men_cipi)
export(men_cisig)
export(men_corr)
export(men_die)
export(men_dot)
export(men_exp)
export(men_hall)
export(men_hist)
export(men_hyp)
export(men_norm)
export(men_parn)
export(men_poi)
export(men_rank)
export(men_regr)
export(men_tab)
export(men_terr)
export(men_time)
export(men_tmu1)
export(men_tmu2)
export(men_tprop)
export(men_vis)
export(mmstat.attrVar)
export(mmstat.axis)
export(mmstat.baraxis)
export(mmstat.dec)
export(mmstat.get)
export(mmstat.getDataNames)
export(mmstat.getDatasets)
export(mmstat.getLog)
export(mmstat.getValue)
export(mmstat.getValues)
export(mmstat.getVar)
export(mmstat.getVarNames)
export(mmstat.getVariableNames)
export(mmstat.lang)
export(mmstat.log)
export(mmstat.math)
export(mmstat.merge)
export(mmstat.plotTestRegions)
export(mmstat.pos)
export(mmstat.range)
export(mmstat.rds)
export(mmstat.round.down)
export(mmstat.round.up)
export(mmstat.set)
export(mmstat.sliderInput)
export(mmstat.ticks)
export(mmstat.ui.call)
export(mmstat.ui.elem)
export(mmstat.ui.update)
export(mmstat.warn)
export(resetpar)
export(stopif)
export(table2dataframe)
export(toRDS)
export(ucfirst)
export(zebra)
export(zzz)
import(shiny)
import(shinydashboardPlus)
importFrom(DT,tableHeader)
importFrom(formatR,tidy_source)
importFrom(highlight,header_html)
importFrom(scatterplot3d,scatterplot3d)
importFrom(shinyWidgets,actionGroupButtons)
importFrom(shinydashboard,dashboardBody)
importFrom(shinydashboard,menuItem)
importFrom(shinydashboard,sidebarMenu)
importFrom(tools,toHTML)
importFrom(utils,browseURL)
importFrom(utils,capture.output)
